#-*- encoding:utf-8 -*-
'''
@author: guoliang@nfschina.com
'''

import pandas as pd
import os
import sys
from pathlib import Path
import glsu
import re

if sys.getdefaultencoding() != 'utf-8':
    reload(sys)
    sys.setdefaultencoding('utf-8')

def installFonts(files):
    res = glsu.checkSudo()
    if res:
        dest = '/usr/local/share/fonts/'
    else: 
        os.system('mkdir -p ~/.local/share/fonts/')
        dest = '~/.local/share/fonts/'   
    try:
        for file in files:
            temp_str = file.split("/")[-1]
            if None == re.match( "^[\w\.-]*$", temp_str, flags=0):
                return "Invalid file!", file

            if not os.path.exists(dest):
                os.makedirs(dest, exist_ok=True)
            cmd = 'cp %s %s' % (file, dest) 
            res = glsu.run(cmd)
            if res != 0:
                return res, cmd
        #-i before cd because cd can't be run directly in sudo
        #glsu.run(['mkfontscale', 'mkfontdir', 'fc-cache -fv'])
        cmd = ['mkfontscale', 'mkfontdir', 'fc-cache -fv', 'rm -f fontsdir', 'rm -f fonts.scale']
        res = glsu.run(cmd)
        return res, cmd        
    except Exception as e:
        return e, cmd
    
def installFontFiles(files):
    res = glsu.checkSudo()
    if res:
        dest = '/usr/local/share/fonts/'
    else: 
        os.system('mkdir -p ~/.local/share/fonts/')
        dest = '~/.local/share/fonts/'   
    try:
        if not os.path.exists(dest):
            os.makedirs(dest, exist_ok=True)

        cmd = 'cp %s %s' % (files, dest) 
        res = glsu.run(cmd)
        if res != 0:
            return res, cmd
        #-i before cd because cd can't be run directly in sudo
        #glsu.run(['mkfontscale', 'mkfontdir', 'fc-cache -fv'])
        cmd = ['mkfontscale', 'mkfontdir', 'fc-cache -fv', 'rm -f fontsdir', 'rm -f fonts.scale']
        res = glsu.run(cmd)
        return res, cmd        
    except Exception as e:
        return e, cmd

def uninstallFonts(files):
    try:
        for file in files:
            cmd = 'rm %s' % file
            user_home = os.path.expanduser('~')
            dest = user_home+'/.local/share/fonts/'
            #TY_os uninstall system fonts need use 'gksu' 
            if os.path.exists("/etc/.systeminfo") or file.startswith(dest) :
                res = glsu.run('rm %s' % file)
            else:
                res = glsu.run('pkexec rm %s' % file)
            if res!=0:
                return res, cmd
        cmd = 'fc-cache -fv'
        res = glsu.run(cmd)
        return res, cmd
    except Exception as e:
        return e, cmd

     
def listFc():
    outputs = os.popen('fc-list -v').read()
    parts = outputs.split('\n\n')
    df = None
    for part in parts:
        font = {}
        if part.strip() == '':
            continue
        try:
            up = part.split('\tcharset:')[0]
#            font['charset'] = part.split('\tcharset:')[1].split('\n(s)\n')[0]
#            down = part.split('\tcharset:')[1].split('\n(s)\n')[1]
        except Exception as e:
            print('{}'.format(e))
            sys.exit(0)
#        updown = up+down
#        lines = updown.split('\n')[1:]
        lines = up.split('\n')[1:]
        for line in lines:
            words = line.split(':')
            key = words[0].strip()
            if( key != 'family' and key != 'style' and key != 'fullname' and key != 'file') :
                continue
            value = ':'.join(words[1:])
            value = value.strip()
            if value.endswith('(s)'): value = value[:-3]
            if value.endswith('(i)'): value = value[:-3]
            if value.startswith('"') and value.endswith('"'): value = value[1:-1]
            value = value.strip()
            #print(key, value)
            font[key] = [value]
        if 'family' in font:
            font['family_'] = font['family'][0].split('"')[0]
        else:
            font['family_'] = 'font without family!'
        if 'style' in font:
            font['style_'] = font['style'][0].split('"')[0]
        else:
            font['style_'] = 'font without style!'
        if 'fullname' in font:
            font['fullname_'] = font['fullname'][0].split('"')[0]
        else:
            font['fullname_'] = 'font without fullname'
        #is symbol link?
        font['is_symlink_'] = Path(font['file'][0]).is_symlink()
        font['file_'] = font['file'][0] if not font['file'][0].startswith('/usr/share/fonts/truetype/') else font['file'][0][len('/usr/share/fonts/truetype/'):]
        df_font = pd.DataFrame.from_dict(font)
        if df is None:
            df = df_font
        else:
            df = pd.concat([df, df_font])
            #df = pd.concat([df, df_font],sort=False)
    df = df.reset_index()        
    return df

if __name__ == '__main__':
    df = listFc()
    #print(df[['family_', 'style', 'fullname', 'file']])
    #print(df[['family_', 'style_', 'fullname_', 'file']])
    df.to_csv('fonts.csv')

        
